package coamonitor;

/**
 * Title:        Corba Object Activity Monitor
 * Description:  Monitor  dell'attivit svolta da oggetti CORBA residenti sui nodi di una rete LAN.
 * Copyright:    Copyright (c) 2001
 * Company:      DEIS
 * @author FMorroia
 * @version 1.0
 */

import com.inprise.vbroker.ObjLocation.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import coamonitor.Interceptor.*;
import org.omg.PortableServer.*;
import com.inprise.vbroker.PortableServerExt.*;
import coamonitor.Monitor.*;
import java.awt.*;


/*Gestore dell'albero */
public class InterceptorTreeManager implements FinderListener {

  private int listenerCount = 0;
  private int listenerNumber = 0;
  private int nodeNumber = 0;
  InterceptorFinder finder;
  private JTree tree;
  private DefaultTreeModel model;
  private DefaultMutableTreeNode root;
  private InterceptorTreeCellRenderer renderer= new InterceptorTreeCellRenderer();
  private ApplicationPanel appPane;
  private org.omg.PortableServer.POA listenerPOA;
  private RootDataManager rootDataManager = new RootDataManager();
  private NodePanel nodePane;
  private org.omg.CORBA.ORB _orb;

  //Costruttori
  public InterceptorTreeManager(org.omg.CORBA.ORB orb, ApplicationPanel ap,NodePanel np) {
    _orb = orb;
    //Inizializzazione dell'albero
    this.initTree();
    appPane = ap;
    nodePane = np;
    this.initListenerPOA(orb);
    //Ricerca dei MonitorBinder presso lo SmartAgent
    finder = new InterceptorFinder(orb,this);
    finder.find();
  }//InterceptorTreeManager

  //inizializzazione dell'albero
  protected void initTree(){
    root = new DefaultMutableTreeNode("Interceptor Node");
    model = new DefaultTreeModel(root);
    tree = new JTree(model);
    int mode = TreeSelectionModel.SINGLE_TREE_SELECTION;
    tree.getSelectionModel().setSelectionMode(mode);
    tree.setCellRenderer(renderer);
  }//initTree

  //inizializzazione del POA dgli InterceptorListener
  void initListenerPOA(org.omg.CORBA.ORB orb){
    try{
      org.omg.PortableServer.POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
      String name = "InterceptorListener";

      BindSupportPolicy bindPolicy = BindSupportPolicyHelper.narrow((((com.inprise.vbroker.PortableServerExt.POA)rootPOA).the_policies())[7]);
      org.omg.CORBA.Any any = orb.create_any();
      BindSupportPolicyValueHelper.insert(any,bindPolicy.value().BY_INSTANCE);

      org.omg.CORBA.Policy[] InterceptorListenerPolicy = {
        rootPOA.create_lifespan_policy(LifespanPolicyValue.PERSISTENT),
        BindSupportPolicyHelper.narrow(orb.create_policy(bindPolicy.policy_type(),any))
      };
      listenerPOA = rootPOA.create_POA(name+"_poa",rootPOA.the_POAManager(),InterceptorListenerPolicy);
      rootPOA.the_POAManager().activate();
    }catch(Exception e){
      e.printStackTrace();
    }
  }//initListenerPOA

  public JTree getTree(){
    return tree;
  }//getTree

  //Implementazione del metodo della FinderListener Interface
  public void interceptorFound(Desc d){
    Desc currentDesc = d;
    TreeData td = new TreeData();
    td.obDesc = currentDesc;
    td.ip = currentDesc.iiop_locator.host;
    td.binder = MonitorBinderHelper.narrow(currentDesc.ref);
    td.dataManager = new InterceptorDataManager(appPane);
    InterceptorListenerServerImpl listenerImpl = new InterceptorListenerServerImpl("InterceptorListener"+listenerCount, listenerPOA);
    listenerImpl.getInterceptorDataManager(td.dataManager);
    td.listener = listenerImpl;
    listenerCount++;
    this.addNode(td);
   }//InterceptroFound

  //aggiunge un nodo all'albero e vi associa i dati contenuti in TreeData
  public void addNode(TreeData td){
    boolean res = false;
    boolean res2 = false;
    String ip = td.ip;
    Enumeration bf = root.breadthFirstEnumeration();
    bf.nextElement();
    if(!bf.hasMoreElements()){

      NodeDataManager nodeDataManager = new NodeDataManager(ip, nodePane, rootDataManager);
      NodeData nodeData = new NodeData();
      nodeData.ip = ip;
      nodeData.nodeDataManager = nodeDataManager;
      DefaultMutableTreeNode node = new DefaultMutableTreeNode(nodeData);
      td.dataManager.setNodeDataManager(nodeDataManager);
      model.insertNodeInto(node,root,root.getChildCount());
      this.addLeaf(node,td);
    }else{
      DefaultMutableTreeNode currentNode = ((DefaultMutableTreeNode)bf.nextElement());
      while((currentNode.getParent().equals(root))&(!res)& bf.hasMoreElements()){
        if(((NodeData)(currentNode.getUserObject())).ip.equals(ip)){
          Enumeration bf2 = currentNode.depthFirstEnumeration();
          while(bf2.hasMoreElements()){
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)bf2.nextElement();
            if(node.getUserObject() instanceof TreeData){
              TreeData data = (TreeData)node.getUserObject();
              if(data.obDesc.iiop_locator.port == td.obDesc.iiop_locator.port){
                res2 = true;
              }
            }
          }
          if(!res2){
            td.dataManager.setNodeDataManager(((NodeData)currentNode.getUserObject()).nodeDataManager);
            this.addLeaf(currentNode,td);
            res = true;
          }
        }
        currentNode = ((DefaultMutableTreeNode)bf.nextElement());
      }
      if(!res & !res2){
        NodeDataManager nodeDataManager = new NodeDataManager(ip, nodePane, rootDataManager);
        NodeData nodeData = new NodeData();
        nodeData.ip = ip;
        nodeData.nodeDataManager = nodeDataManager;
        DefaultMutableTreeNode node = new DefaultMutableTreeNode(nodeData);
        td.dataManager.setNodeDataManager(nodeDataManager);
        model.insertNodeInto(node,root,root.getChildCount());
        this.addLeaf(node,td);
      }
    }
  }//addNode

  /**Aggiunge un nodo Foglia al nodo node associandogli TreeData*/
  void addLeaf(DefaultMutableTreeNode node, TreeData td){
    DefaultMutableTreeNode newNode = new DefaultMutableTreeNode();
    newNode.setUserObject(td);
    newNode.setAllowsChildren(false);
    model.insertNodeInto(newNode,node,node.getChildCount());
    TreeNode[] nodes = model.getPathToRoot(newNode);
    TreePath path = new TreePath(nodes);
    tree.makeVisible(path);
  }//addNode

  /**metodo di rimozione del Listaner dall'Interceptor corrispondente a node*/
  public void removeListener(DefaultMutableTreeNode node){
    if(node.getUserObject() instanceof TreeData){
      TreeData data = (TreeData)node.getUserObject();
      if(data.registered){
        try{
          MonitorBinder binder = data.binder;
          binder.removeListener();
          data.registered = false;
          listenerNumber--;
        }catch(Exception e){
          this.appShutDown(((TreeData)node.getUserObject()).obDesc);
        }
      }
    }
  }//removeNode

  /**metodo di rimozione di tuti i Listener usato solo alla terminazionwe
   * usatp solo alla terminazione del Monitor
   */
  public void removeAllListener(){
    int i = 1;
    Enumeration bf = root.depthFirstEnumeration();
    DefaultMutableTreeNode currentNode = null;
    while(bf.hasMoreElements()){
      currentNode = (DefaultMutableTreeNode)bf.nextElement();
      if(currentNode.getUserObject() instanceof TreeData){
        this.removeListener(currentNode);
      }
    }
  }//removeAllListener

  /**abilitazione di un nuovo InterceptorDataManager e disabilitazione del vecchio*/
  public void enableInterceptorDataManager(TreeData td){
    td.dataManager.appPane.removeInterceptorDataManager();
    td.dataManager.appPane.setInterceptorDataManager(td.dataManager);
  }//startMonitor

  /**abilitazione di un nuovo NodeDataManager e disabilitazione del vecchio*/
  public void enableNodeDataManager(NodeData nd){
    nd.nodeDataManager.nodePanel.removeNodeDataManager();
    nd.nodeDataManager.nodePanel.setNodeDataManager(nd.nodeDataManager);
  }//enableNodeDataManager

  //registarzione di un Listener presso l'Interceptor specificato in TreeData
  public void register(TreeData td){
    //Controllo sul numero di applicazioni registrate
    if(!(listenerNumber>=10)){
      InterceptorListener listener = td.listener.getReference();
      td.binder.addListener(listener);
      if(!td.registered){
        listenerNumber++;
      }
      td.registered = true;
    }else{
      appPane.jMessageArea.append("The Monitor is full\n");
    }
  }//register

  //rimozione grafica di un nodo dall'albero
  public void removeNode(DefaultMutableTreeNode dmtn){
    java.lang.Object thisObject = dmtn.getUserObject();
    if(thisObject instanceof TreeData){
        TreeData currentTreeData = (TreeData)thisObject;
        appPane.jMessageArea.append("Appliction "+currentTreeData.obDesc.instance_name+"ended, node removed\n");
        currentTreeData = null;
    }else{
      if(thisObject.getClass().toString().equals("class coamonitor.NodeData")){
        NodeData currentNodeData = (NodeData)thisObject;
        currentNodeData = null;
      }
    }
    model.removeNodeFromParent(dmtn);
  }//removeNode

  /**metodo invocato in conseguenza della caduta di un'applicazione*/
  public void appShutDown(Desc d){
  DefaultMutableTreeNode currentNode = null;
  Enumeration bf = root.breadthFirstEnumeration();
    while(bf.hasMoreElements()){
      currentNode = (DefaultMutableTreeNode)bf.nextElement();
      java.lang.Object thisObject = currentNode.getUserObject();
      if(thisObject instanceof TreeData){
        TreeData td= (TreeData)currentNode.getUserObject();
        if(td.obDesc.iiop_locator.toString().equals(d.iiop_locator.toString())){
          DefaultMutableTreeNode parent = (DefaultMutableTreeNode)currentNode.getParent();
            this.removeNode(currentNode);
          if(parent.getChildCount() == 0){
            this.removeNode(parent);
          }
        }
      }
    }
  }//appShutDown

  /**Metodo dell'interfaccia FinxderLiastener Interface*/
  public void deleteInterceptor(Desc d){
    this.appShutDown(d);
  }//deleteinterceptor
}//InterceptorTreeManager

/**Classe interceptorTreeCellrender, ridefinisce l'aspetto dei nodi
 dell'albero */

class InterceptorTreeCellRenderer extends DefaultTreeCellRenderer{
  public Component getTreeCellRendererComponent(JTree tree,
    Object value, boolean selected, boolean expanded,
    boolean leaf, int row, boolean hasFocus){

    super.getTreeCellRendererComponent(tree, value, selected,
    expanded, leaf, row, hasFocus);

    DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
    Object thisObject = node.getUserObject();
    if(thisObject instanceof TreeData){
      this.setText(((TreeData)thisObject).obDesc.instance_name);
    }else{
      if(thisObject instanceof NodeData){
        this.setText(((NodeData)thisObject).ip);
      }
    }
    return this;
  }//getTreeCellRendererComponent
}//InterceptorTreeCellRenderer
